package org.artifactory.queue.behaviors;

import com.google.common.collect.Lists;
import org.artifactory.api.events.EventHintType;
import org.artifactory.storage.event.EventInfo;
import org.testng.annotations.Test;

import java.util.List;

import static org.artifactory.storage.event.EventType.create;
import static org.artifactory.storage.event.EventType.delete;
import static org.artifactory.storage.event.EventType.props;
import static org.artifactory.storage.event.EventType.update;
import static org.testng.Assert.assertEquals;

@Test
public class QueueReductionBehaviourTest extends AbstractEventReductionBehaviorTest {

    public void chainBasicBehaviours() {
        QueueReductionBehaviours behaviors = QueueReductionBehaviours.Basic;

        List<EventInfo> input;
        List<EventInfo> output;

        input = Lists.newArrayList();
        add(input, create, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        add(input, delete, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        output = behaviors.applyChain(input);
        assertEquals(output.size(), 0);

        input = Lists.newArrayList();
        add(input, delete, "a"); //because of CreateEliminateDeleteBehaviour
        add(input, create, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        add(input, delete, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        output = behaviors.applyChain(input);
        assertEquals(output.size(), 0);

        reset();
        input = Lists.newArrayList();
        add(input, create, "a"); //removed 2nd because of BasicEventDedupBehaviour
        add(input, delete, "a"); //removed 1st because of CreateEliminateDeleteBehaviour
        add(input, create, "a");
        output = behaviors.applyChain(input);
        assertEquals(output.size(), 1);
        assertEquals(output.get(0).toString(), "3|create|a");
    }


    public void chainIncrementalIndexBehaviours() {
        QueueReductionBehaviours behaviors = QueueReductionBehaviours.IncrementalIndex;

        List<EventInfo> input;
        List<EventInfo> output;

        input = Lists.newArrayList();
        add(input, create, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        add(input, delete, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        output = behaviors.applyChain(input);
        assertEquals(output.size(), 0);

        input = Lists.newArrayList();
        add(input, delete, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        add(input, create, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        add(input, delete, "a");
        output = behaviors.applyChain(input);
        assertEquals(output.size(), 1);
        assertEquals(output.get(0).toString(), "3|delete|a");

        reset();
        input = Lists.newArrayList();
        add(input, create, "a");
        add(input, delete, "a"); //removed 1st because of DeleteEliminateUpdateOrCreateBehavior
        add(input, create, "a"); //removed 2nd because of FederatedDedupBehavior
        output = behaviors.applyChain(input);
        assertEquals(output.size(), 1);
        assertEquals(output.get(0).toString(), "3|create|a");

        reset();
        input = Lists.newArrayList();
        add(input, update, "b");
        add(input, create, "a");
        add(input, props, "a");
        add(input, props, "a");
        add(input, props, "b");
        add(input, update, "a");
        add(input, update, "a");
        add(input, props, "a");
        output = behaviors.applyChain(input);
        assertEquals(output.size(), 2);
        assertEquals(output.get(0).toString(), "5|update|b", "regular prioritize dedup");
        assertEquals(output.get(1).toString(), "8|create|a", "merged prioritize dedup");
        assertEquals(output.get(1).getHint(EventHintType.RollbackEventId), "2");
    }

}
