package org.artifactory.version.converter.v312;

import lombok.NonNull;
import org.artifactory.version.converter.XmlConverter;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.Namespace;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Volodymyr Shulga
 */
public class AddBasicAuthenticationEnabledConverter implements XmlConverter {
    private static final Logger log = LoggerFactory.getLogger(AddBasicAuthenticationEnabledConverter.class);

    @Override
    public void convert(@NonNull Document doc) {
        String simpleName = this.getClass().getSimpleName();
        log.info("Started {}", simpleName);
        Element rootElement = doc.getRootElement();
        Namespace namespace = rootElement.getNamespace();
        Element securityElement = rootElement.getChild("security", namespace);
        if (securityElement != null) {
            Element basicAuthEnabledElement = securityElement.getChild("basicAuthEnabled", namespace);
            if (basicAuthEnabledElement == null) {
                basicAuthEnabledElement = new Element("basicAuthEnabled", namespace);
                // basic auth should remain enabled for existing installations.
                basicAuthEnabledElement.addContent("true");
                securityElement.addContent(basicAuthEnabledElement);
            }
        }
        log.info("Finished {}", simpleName);
    }
}