package org.artifactory.storage.db.event.service;


import org.artifactory.storage.db.event.entity.enums.NodeEventCursorStatus;
import org.artifactory.storage.db.event.entity.enums.NodeEventCursorType;
import org.artifactory.storage.db.event.model.NodeEventCursor;

import javax.annotation.Nonnull;
import java.util.List;

/**
 * @author Uriah Levy
 * A bussiness service that manages the node events cursor table. This service
 * keeps track and updates an indepedant cursor for the different event-based operations,
 * based on a unique operation identifier provided by the caller.
 */
public interface NodeEventCursorService {

    /**
     * Insert a new event operator cursor
     *
     * @param operationId the operation Id of the new cursor
     */
    void insertCursor(String operationId, long marker, NodeEventCursorType nodeEventCursorType);

    /**
     * Insert a new event operator cursor
     *
     */
    void insertCursor(@Nonnull NodeEventCursor nodeEventCursor);

    /**
     * Update the current cursor position.
     *
     * @param eventCursor the {@link NodeEventCursor} to set
     */
    void updateOrInsertCursor(NodeEventCursor eventCursor);

    /**
     * Get the current {@link NodeEventCursor}
     *
     * @param operationId the identifier of the event operation (metadata, replication, etc)
     * @return the current cursor position.
     */
    NodeEventCursor cursorForOperator(String operationId);

    void deleteCursor(String operationId);

    long oldestTimestampForEventLogOperator();

    /**
     * Update the cursor by ID
     */
    void updateCursor(String operatorId, long marker);

    /**
     * Get all cursors by status
     *
     * @param status the status of cursor(synced, out_of_synced)
     * @return List of cursors with the same status
     */
    List<NodeEventCursor> getAllCursorsByStatus(@Nonnull NodeEventCursorStatus status,
            @Nonnull NodeEventCursorType type);

    /**
     * Get all cursors that using event id and less updated than timestamp
     *
     * @param status the status of cursor(synced, out_of_synced)
     * @return List of cursors
     */
    List<NodeEventCursor> getEventIdCursorsBy(@Nonnull NodeEventCursorStatus status, long timestamp);

    /**
     * change status for cursor
     *
     * @param status update cursor to status
     */
    void updateCursorWithStatus(@Nonnull String cursorOpId, @Nonnull NodeEventCursorStatus status);

    /**
     * update the marker with last updated 'now' if update boolean is true
     */
    void updateCursorMarker(@Nonnull String cursorName, long marker, boolean promoteLastUpdated);
}