package org.artifactory.ui.rest.model.admin.configuration.repository.federation;

import com.fasterxml.jackson.annotation.JsonTypeName;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.artifactory.descriptor.config.MutableCentralConfigDescriptor;
import org.artifactory.descriptor.repo.FederatedRepoDescriptor;
import org.artifactory.descriptor.repo.RepoDescriptor;
import org.artifactory.rest.common.exception.RepoConfigException;
import org.artifactory.rest.common.util.JsonUtil;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.CommonLocalRepositoryConfigModel;
import org.artifactory.ui.rest.service.admin.configuration.repositories.util.CreateRepoConfigHelper;
import org.artifactory.ui.rest.service.admin.configuration.repositories.util.UpdateRepoConfigHelper;
import org.artifactory.ui.rest.service.admin.configuration.repositories.util.builder.RepoConfigDescriptorBuilder;
import org.artifactory.ui.rest.service.admin.configuration.repositories.util.builder.RepoConfigModelBuilder;
import org.artifactory.ui.rest.service.admin.configuration.repositories.util.validator.RepoConfigValidator;

/**
 * @author AndreiK.
 */
@Data
@EqualsAndHashCode(callSuper = false)
@JsonTypeName("federatedRepoConfig")
public class FederatedRepositoryConfigModel extends CommonLocalRepositoryConfigModel {

    protected FederationConfigModel federation;
    protected FederatedRepoAdvancedRepositoryConfigModel advanced;

    @Override
    public FederatedRepoAdvancedRepositoryConfigModel getAdvanced() {
        return advanced;
    }

    @Override
    public MutableCentralConfigDescriptor createRepo(CreateRepoConfigHelper creator,
            MutableCentralConfigDescriptor configDescriptor) throws RepoConfigException {
        return creator.handleFederated(this, configDescriptor);
    }

    @Override
    public void updateRepo(UpdateRepoConfigHelper updater) throws RepoConfigException {
        updater.handleFederated(this);
    }

    @Override
    public FederatedRepoDescriptor toDescriptor(RepoConfigValidator validator, RepoConfigDescriptorBuilder builder,
            MutableCentralConfigDescriptor configDescriptor)
            throws RepoConfigException {
        validator.validateFederated(this);
        return builder.buildFederatedDescriptor(this);
    }

    @Override
    public FederatedRepositoryConfigModel fromDescriptor(RepoConfigModelBuilder builder, RepoDescriptor descriptor) {
        builder.populateFederatedDescriptorValuesToModel((FederatedRepoDescriptor) descriptor, this);
        return this;
    }

    @Override
    public String toString() {
        return JsonUtil.jsonToString(this);
    }
}
